package com.sun.media.sound;

import java.util.List;
import javax.sound.sampled.LineListener;
import javax.sound.midi.MetaEventListener;
import javax.sound.midi.ControllerEventListener;
import javax.sound.midi.ShortMessage;
import javax.sound.midi.MetaMessage;
import javax.sound.sampled.LineEvent;
import java.util.ArrayList;

final class EventDispatcher implements Runnable
{
    private static final int AUTO_CLOSE_TIME = 5000;
    private final ArrayList eventQueue;
    private Thread thread;
    private final ArrayList<EventDispatcher.ClipInfo> autoClosingClips;
    private final ArrayList<EventDispatcher.LineMonitor> lineMonitors;
    static final int LINE_MONITOR_TIME = 400;
    
    EventDispatcher() {
        this.eventQueue = new ArrayList();
        this.thread = null;
        this.autoClosingClips = new ArrayList();
        this.lineMonitors = new ArrayList();
    }
    
    synchronized void start() {
        if (this.thread == null) {
            this.thread = JSSecurityManager.createThread(this, "Java Sound Event Dispatcher", true, -1, true);
        }
    }
    
    void processEvent(EventDispatcher.EventInfo eventInfo) {
        int listenerCount = eventInfo.getListenerCount();
        if ((eventInfo.getEvent() instanceof LineEvent)) {
            LineEvent lineEvent = (LineEvent)eventInfo.getEvent();
            for (int k = 0; k < listenerCount; ++k) {
                try {
                    ((LineListener)eventInfo.getListener(k)).update(lineEvent);
                }
                catch (Throwable t3) {}
            }
            return;
        }
        if ((eventInfo.getEvent() instanceof MetaMessage)) {
            MetaMessage metaMessage = (MetaMessage)eventInfo.getEvent();
            for (int j = 0; j < listenerCount; ++j) {
                try {
                    ((MetaEventListener)eventInfo.getListener(j)).meta(metaMessage);
                }
                catch (Throwable t2) {}
            }
            return;
        }
        if ((eventInfo.getEvent() instanceof ShortMessage)) {
            ShortMessage shortMessage = (ShortMessage)eventInfo.getEvent();
            if ((shortMessage.getStatus() & 0xF0) == 0xB0) {
                for (int i = 0; i < listenerCount; ++i) {
                    try {
                        ((ControllerEventListener)eventInfo.getListener(i)).controlChange(shortMessage);
                    }
                    catch (Throwable t) {}
                }
            }
            return;
        }
        else {
            Printer.err("Unknown event type: " + eventInfo.getEvent());
            return;
        }
    }
    
    void dispatchEvents() {
        EventDispatcher.EventInfo eventInfo = null;
        synchronized (this) {
            try {
                if (this.eventQueue.size() == 0) {
                    if (this.autoClosingClips.size() > 0 || this.lineMonitors.size() > 0) {
                        int n = 5000;
                        if (this.lineMonitors.size() > 0) {
                            n = 400;
                        }
                        this.wait((long)n);
                    }
                    else {
                        this.wait();
                    }
                }
            }
            catch (InterruptedException ex) {}
            if (this.eventQueue.size() > 0) {
                eventInfo = (EventDispatcher.EventInfo)this.eventQueue.remove(0);
            }
        }
        if (eventInfo != null) {
            this.processEvent(eventInfo);
        }
        else {
            if (this.autoClosingClips.size() > 0) {
                this.closeAutoClosingClips();
            }
            if (this.lineMonitors.size() > 0) {
                this.monitorLines();
            }
        }
    }
    
    private synchronized void postEvent(EventDispatcher.EventInfo e) {
        this.eventQueue.add(e);
        this.notifyAll();
    }
    
    @Override
    public void run() {
        while (true) {
            try {
                this.dispatchEvents();
                continue;
            }
            catch (Throwable t) {}
        }
    }
    
    void sendAudioEvents(Object o, List list) {
        if (list == null || list.size() == 0) {
            return;
        }
        this.start();
        this.postEvent(new EventDispatcher.EventInfo(o, list));
        return;
    }
    
    private void closeAutoClosingClips() {
        synchronized (this.autoClosingClips) {
            long currentTimeMillis = System.currentTimeMillis();
            for (int i = this.autoClosingClips.size() - 1; i >= 0; --i) {
                EventDispatcher.ClipInfo clipInfo = (EventDispatcher.ClipInfo)this.autoClosingClips.get(i);
                if (clipInfo.isExpired(currentTimeMillis)) {
                    AutoClosingClip clip = clipInfo.getClip();
                    if (!clip.isOpen() || !clip.isAutoClosing()) {
                        this.autoClosingClips.remove(i);
                    }
                    else if (!clip.isRunning() && !clip.isActive() && clip.isAutoClosing()) {
                        clip.close();
                    }
                }
            }
        }
    }
    
    private int getAutoClosingClipIndex(AutoClosingClip autoClosingClip) {
        synchronized (this.autoClosingClips) {
            int i = this.autoClosingClips.size() - 1;
            while (i >= 0) {
                if (autoClosingClip.equals(((EventDispatcher.ClipInfo)this.autoClosingClips.get(i)).getClip())) {
                    return i;
                }
                --i;
            }
        }
        return -1;
    }
    
    void autoClosingClipOpened(AutoClosingClip autoClosingClip) {
        int autoClosingClipIndex = 0;
        synchronized (this.autoClosingClips) {
            autoClosingClipIndex = this.getAutoClosingClipIndex(autoClosingClip);
            if (autoClosingClipIndex == -1) {
                this.autoClosingClips.add(new EventDispatcher.ClipInfo(autoClosingClip));
            }
        }
        if (autoClosingClipIndex == -1) {
            synchronized (this) {
                this.notifyAll();
            }
        }
    }
    
    void autoClosingClipClosed(AutoClosingClip autoClosingClip) {
        synchronized (this.autoClosingClips) {
            int autoClosingClipIndex = this.getAutoClosingClipIndex(autoClosingClip);
            if (autoClosingClipIndex != -1) {
                this.autoClosingClips.remove(autoClosingClipIndex);
            }
        }
    }
    
    private void monitorLines() {
        synchronized (this.lineMonitors) {
            for (int i = 0; i < this.lineMonitors.size(); ++i) {
                ((EventDispatcher.LineMonitor)this.lineMonitors.get(i)).checkLine();
            }
        }
    }
    
    void addLineMonitor(EventDispatcher.LineMonitor lineMonitor) {
        synchronized (this.lineMonitors) {
            if (this.lineMonitors.indexOf(lineMonitor) >= 0) {
                return;
            }
            this.lineMonitors.add(lineMonitor);
        }
        synchronized (this) {
            this.notifyAll();
        }
    }
    
    void removeLineMonitor(EventDispatcher.LineMonitor lineMonitor) {
        synchronized (this.lineMonitors) {
            if (this.lineMonitors.indexOf(lineMonitor) < 0) {
                return;
            }
            this.lineMonitors.remove(lineMonitor);
        }
    }
    
    private class EventInfo
    {
        private final Object event;
        private final Object[] listeners;
        
        EventInfo(Object event, List list) {
            this.event = event;
            this.listeners = list.toArray();
        }
        
        Object getEvent() {
            return this.event;
        }
        
        int getListenerCount() {
            return this.listeners.length;
        }
        
        Object getListener(int n) {
            return this.listeners[n];
        }
    }
    
    private class ClipInfo
    {
        private final AutoClosingClip clip;
        private final long expiration;
        
        ClipInfo(AutoClosingClip clip) {
            this.clip = clip;
            this.expiration = System.currentTimeMillis() + 5000L;
        }
        
        AutoClosingClip getClip() {
            return this.clip;
        }
        
        boolean isExpired(long n) {
            return n > this.expiration;
        }
    }
    
    interface LineMonitor
    {
        public abstract void checkLine();
    }
}
